#uses "fwAccessControl/fwAccessControl.ctc"
#uses "fwAccessControl/fwAccessControl_Egroups.ctc"
#uses "fwUnitTestComponentAsserts.ctl"


void assertExceptionInfo(string errSeverity, string errString, dyn_string exceptionInfo)
{
    // until Paul fixes the unit-testing framework's assertError() we need this one:
    if (dynlen(exceptionInfo) <3) {
	assert(false,"No exceptionInfo while expecting to have one");
	return;
    }
    assertError(errSeverity,errString,exceptionInfo);
}



private string getErrorString(int errCode)
{
    // convert a integer error code to a 5-digit string
    // as needed by getCatStr()
    string errCodeAsString;
    sprintf(errCodeAsString,"%05d",errCode);
    string errCat="_errors";
    string errString=getCatStr(errCat,errCodeAsString);
    return errString;
}

// extract the values of a specific key for all entries of a dyn_mapping
dyn_mixed dynMappingExtractByKey(dyn_mapping dm, mixed key)
{
    dyn_mixed retval;
    mixed uninitialized;
    for (int i=1;i<=dynlen(dm);i++) {
	if (mappingHasKey(dm[i],key)) retval[i]=(dm[i])[key]; else retval[i]=uninitialized;
    }
    return retval;
}

void test_fwAccessControl_Egroups_setupSuite() 
{
}

void test_fwAccessControl_Egroups_teardownSuite() 
{
}

void test_fwAccessControl_Egroups_getMatchingPattern()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="test-fwAC*";
    string attrNames="";
    string childType="";
    dyn_string otherFilters;

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);
    assertEmpty(exceptionInfo, "Exceptions have been raised");

    dyn_string allCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allCNs,"test-fwACConfig-TestCV")>0,"Configuration egroup not found");
    assertTrue(dynContains(allCNs,"test-fwACDomain-MyDCS")>0,"Domain egroup not found");
    assertTrue(dynContains(allCNs,"test-fwACPrivilege-MyDCS-1-Monitor")>0,"Privilege egroup not found");
    assertTrue(dynContains(allCNs,"test-fwACRole-TestCV-expert")>0,"Role egroup not found");
}

void test_fwAccessControl_Egroups_getAllTestConfigurations()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="";
    string childType="CONFIGURATIONS";
    string attrNames="";
    dyn_string otherFilters;//=makeDynString("cn=test-fwAC*"); // we put a special filter for this test to help the LDAP server

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);

    assertEmpty(exceptionInfo, "Exceptions have been raised");

    dyn_string allCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allCNs,"test-fwACConfig-TestCV")>0,"Configuration egroup not found");
}


void test_fwAccessControl_Egroups_getConfigurationDomains()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="test-fwACConfig-TestCV";
    string childType="DOMAINS";
    string attrNames="adminDescription,memberOf";
    dyn_string otherFilters;

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);

    assertEmpty(exceptionInfo, "Exceptions have been raised");

    dyn_string allDomainCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allDomainCNs,"test-fwACDomain-MyDCS")>0,"Domain egroup not found");
    assertTrue(dynContains(allDomainCNs,"test-fwACDomain-TestCV")>0,"Domain egroup not found");
}


void test_fwAccessControl_Egroups_getDomainPrivileges()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="test-fwACDomain-MyDCS";
    string childType="PRIVILEGES";
    string attrNames="adminDescription,memberOf";
    dyn_string otherFilters;

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);
    assertEmpty(exceptionInfo, "Exceptions have been raised");

    //DebugTN(result);


    dyn_string allPrivilegeCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allPrivilegeCNs,"test-fwACPrivilege-MyDCS-2-Control")>0,"Privilege egroup not found");
    assertTrue(dynContains(allPrivilegeCNs,"test-fwACPrivilege-MyDCS-3-Debug")>0,"Privilege egroup not found");

}

void test_fwAccessControl_Egroups_getPrivilegeHolders()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="test-fwAC*";
    string childType="ROLES";
    string attrNames;
    dyn_string otherFilters=makeDynString("member=cn=test-fwACPrivilege-TestCV-1-monitor,"+fwAccessControl_egroupsBaseDN);

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);
    assertEmpty(exceptionInfo, "Exceptions have been raised");

    dyn_string allRoleCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allRoleCNs,"test-fwACRole-TestCV-expert")>0,"Privilege egroup not found");
    assertTrue(dynContains(allRoleCNs,"test-fwACRole-TestCV-operator")>0,"Privilege egroup not found");
}


void test_fwAccessControl_Egroups_getConfigurationRoles()
{
    dyn_string exceptionInfo;

    string egroupNameFilter="test-fwACConfig-TestCV";
    string childType="ROLES";
    string attrNames="adminDescription,memberOf";
    dyn_string otherFilters;

    dyn_mapping result = fwAccessControl_getEgroupSubgroups(egroupNameFilter, exceptionInfo, attrNames, childType, otherFilters);

    assertEmpty(exceptionInfo, "Exceptions have been raised");

//    DebugTN(result);

    dyn_string allRoleCNs=dynMappingExtractByKey(result, "cn");
    assertTrue(dynContains(allRoleCNs,"test-fwACRole-TestCV-operator")>0,"Test role egroup not found");
    assertTrue(dynContains(allRoleCNs,"test-fwACRole-TestCV-expert")>0,"Test role egroup not found");

}


void test_fwAccessControl_Egroups_getDomainFromEgroup_nonExisting()
{
    dyn_string exceptionInfo;

    string domainEgroup="some-non-existing-egroup";
    string domainName, domainComment;
    dyn_string privilegeEgroups, privilegeNames;

    fwAccessControl_getDomainFromEgroup(domainEgroup, domainName, domainComment, privilegeEgroups, privilegeNames, exceptionInfo);
    DebugTN(exceptionInfo);
    assertTrue(dynlen(exceptionInfo),"Got no exception while expected one.");
}


void test_fwAccessControl_Egroups_getDomainFromEgroup_withWildcard()
{
    dyn_string exceptionInfo;

    string domainEgroup="test-fwAC*";
    string domainName, domainComment;
    dyn_string privilegeEgroups, privilegeNames;

    fwAccessControl_getDomainFromEgroup(domainEgroup, domainName, domainComment, privilegeEgroups, privilegeNames, exceptionInfo);
    DebugTN(exceptionInfo);
    assertTrue(dynlen(exceptionInfo),"Got no exception while expected one.");
}


void test_fwAccessControl_Egroups_getDomainFromEgroup_wrongType()
{
    dyn_string exceptionInfo;

    string domainEgroup="test-fwACRole-TestCV-expert";
    string domainName, domainComment;
    dyn_string privilegeEgroups, privilegeNames;

    fwAccessControl_getDomainFromEgroup(domainEgroup, domainName, domainComment, privilegeEgroups, privilegeNames, exceptionInfo);
    DebugTN(exceptionInfo);
    assertTrue(dynlen(exceptionInfo),"Got no exception while expected one.");
}

void test_fwAccessControl_Egroups_getDomainFromEgroup()
{
    dyn_string exceptionInfo;

    string domainEgroup="test-fwACDomain-MyDCS";
    string domainName, domainComment;
    dyn_string privilegeEgroups, privilegeNames;

    fwAccessControl_getDomainFromEgroup(domainEgroup, domainName, domainComment, privilegeEgroups, privilegeNames, exceptionInfo);

    //DebugTN("Domain egroup",domainEgroup);
    //DebugTN("Domain name",domainName);
    //DebugTN("Domain comment", domainComment);
    //DebugTN("Privilege egroups",privilegeEgroups);
    //DebugTN("Privilege names",privilegeNames);

    assertEmpty(exceptionInfo, "Exceptions have been raised");
    assertEqual("MyDCS",domainName,"Domain name not correct");
    assertEqual(makeDynString("Monitor","Control","Debug","Modify","Administer"), privilegeNames,"Privileges wrong");
    assertTrue(strlen(domainComment)>0,"Domain comment empty");
}

void test_fwAccessControl_Egroups_getRoleFromEgroup()
{
    dyn_string exceptionInfo;
    string roleEgroup="test-fwACRole-TestCV-expert";
    string roleName;
    string roleComment;
    dyn_string accessRights;
    dyn_string privilegeEgroups, memberEgroups;
    dyn_string expectedAccessRights=makeDynString("TestCV:operator","TestCV:expert","TestCV:monitor");
    dyn_string expectedMemberEgroups=makeDynString("test-fwACGroup-SomeGroup1","test-fwACGroup-SomeGroup2");

    fwAccessControl_getRoleFromEgroup(roleEgroup, roleName, roleComment, accessRights, privilegeEgroups, memberEgroups, exceptionInfo);

    //DebugTN("RoleEgroup", roleEgroup);
    //DebugTN("RoleName", roleName);
    //DebugTN("RoleComment",roleComment);
    //DebugTN("Access Rights", accessRights);
    //DebugTN("Privilege Egroups", privilegeEgroups);
    //DebugTN("Member Egroups", memberEgroups);

    assertEmpty(exceptionInfo, "Exceptions have been raised");
    assertEqual("TestCV-expert",roleName);

    dynSortAsc(expectedAccessRights);
    dynSortAsc(accessRights);
    assertEqual(expectedAccessRights,accessRights);

    dynSortAsc(expectedMemberEgroups);
    dynSortAsc(memberEgroups);
    assertEqual(expectedMemberEgroups,memberEgroups);
}


void test_fwAccessControl_egroupToAccessRight()
{
    dyn_string exceptionInfo;
    string egroupName="test-fwACPrivilege-MyDCS-3-Debug";
    string accessRight;
    int privNumber;
    fwAccessControl_egroupToAccessRight(egroupName, accessRight, privNumber, exceptionInfo);

    //DebugTN("egroupName",egroupName);
    //DebugTN("accessRight",accessRight);
    //DebugTN("privNumber",privNumber);

    assertEmpty(exceptionInfo, "Exceptions have been raised");
    assertEqual("MyDCS:Debug",accessRight);
    assertEqual(3,privNumber);
}

void test_fwAccessControl_accessRightToEgroup()
{
    dyn_string exceptionInfo;
    string accessRight="MyDCS:Debug";
    string egroup=fwAccessControl_accessRightToEgroup(accessRight, exceptionInfo);

    //DebugTN("AccessRight",accessRight);
    //DebugTN("Egroup",egroup);

    assertEmpty(exceptionInfo, "Exceptions have been raised");
    assertEqual("test-fwACPrivilege-MyDCS-3-Debug",egroup);
}


void test_fwAccessControl_domainNameToEgroup()
{
    dyn_string exceptionInfo;
    string domainName="MyDCS";
    string egroup=fwAccessControl_domainNameToEgroup(domainName, exceptionInfo);

    //DebugTN("DomainName",domainName);
    //DebugTN("Egroup",egroup);

    assertEmpty(exceptionInfo, "Exceptions have been raised");
    assertEqual("test-fwACDomain-MyDCS",egroup);

}

