/**@file testFwAccessControl.ctl

@brief Test Suite for JCOP Framework Access Control
@par Creation Date 23/06/2014
*/

#uses "fwUnitTestComponentAsserts.ctl" 
#uses "fwAccessControl/fwAccessControl.ctc" // The library of code being tested.

const string TestDomainName="TEST";

/**
@brief This routine is optional.
Prepare the appropriate global environment for all of the test cases in this suite.
setupSuite() is called once only, before any of the test cases are called.
*/ 
void testFwAccessControl_setupSuite()
{
//	DebugTN("testFwAccessControl_setupSuite");
	dyn_string exceptionInfo;
	fwAccessControl_checkAddDomain(TestDomainName,makeDynString("Visualize","Operate","Expert","Admin"),exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN("Problem in testFwAccessControl_setupSuite",exceptionInfo[2]);};
}

//-------------------------------------------------------

/**
@brief testExampleComponent_setup() ensures a consistent environment exists prior to calling each test case.
This routine is optional. If you declare one here, it should prepare any appropriate environment
that will be required before running each of the test cases in this suite.

*/ 
void testFwAccessControl_setup()
{
//	DebugTN("testFwAccessControl_setup");
}

//-------------------------------------------------------

/**
@brief Example of a developer's test routine. 
A test*.ctl file can contain as many test routines as the developer wants.
*/ 
void testAccessControl_testIsGrantedSystemVisualize()
{
    dyn_string exceptionInfo;
    bool granted;
    fwAccessControl_isGranted("SYSTEM:Visualize",granted,exceptionInfo);
    assertEmpty(exceptionInfo);
    assertTrue(granted);
}


void testAccessControl_testIsGrantedTestVisualize()
{
    dyn_string exceptionInfo;
    bool granted;
    fwAccessControl_isGranted(TestDomainName+":Visualize",granted,exceptionInfo);
    assertEmpty(exceptionInfo);
    assertFalse(granted);
}


//-------------------------------------------------------

/**
@brief This routine is optional. 
Called after each test case routine.
*/ 
void testFwAccessControl_teardown()
{
//	DebugTN("testFwAccessControl_teardown");
}

//-------------------------------------------------------

/**
@brief This routine is optional. 
Called after all test case routines in this file have been run.
*/ 
void testFwAccessControl_teardownSuite()
{
//	DebugTN("testFwAccessControl_teardownSuite");
	dyn_string exceptionInfo, domainList, domainFullNames;
	fwAccessControl_getAllDomains(domainList,domainFullNames,exceptionInfo);
	if (dynlen(exceptionInfo)) { DebugTN("Problem in testFwAccessControl_teardownSuite",exceptionInfo[2]);};
	if (dynContains(domainList,TestDomainName)) {
	    fwAccessControl_deleteDomain(TestDomainName,exceptionInfo);
	    if (dynlen(exceptionInfo)) { DebugTN("Problem in testFwAccessControl_teardownSuite",exceptionInfo[2]);};
	}
}
